//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_CLIP_PATH_INCLUDED
#include "svg_filter.h"
#include "svg_fe_alpha_mask.h"
#define SVG_CLIP_PATH_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// clip_path: in fact, a specialized filter, which creates a "bw" alpha
//			  mask from its child shapes, and uses it to filter the
//			  clipping image.
//////////////////////////////////////////////////////////////////////
class clip_path : public filter 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		clip_path(element *parent, unsigned level): filter(parent, level)
		{}

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "clipPathUnits") == 0) 
			{	
				if (strcmp(value, "objectBoundingBox")==0) m_relative_content_units = 1;
				if (strcmp(value, "userSpaceOnUse")==0) m_relative_content_units = 0;
			}
			else
			{//if the style is not recognized:
				filter::on_attribute(prs,name,value);
			}
		}

		//no extent is needed:
		virtual double extent_ratio() {return 1.0;}

		virtual const char * tag() {return "clipPath";}; 

		//used to add to child elements one additional, artifical filter primitive:
		//fe_clip
		virtual void commit(parser &prs) 
		{
			fe_clip::create(prs, this,level()+1);
			filter::commit(prs);
		}

	protected:

}; //clip_path

}; //namespace svg
#endif //SVG_CLIP_PATH_INCLUDED

