//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_ELLIPSE_INCLUDED
#include "svg_path.h"
#define SVG_ELLIPSE_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// ellipse: draws an ellipse
//////////////////////////////////////////////////////////////////////
class ellipse : public path
{

	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		ellipse(element *parent, unsigned level): path(parent, level), 
		m_cx(0), m_cy(0), m_rx(0), m_ry(0)
		{}
	
		virtual const char * tag() 	{ return "ellipse";	}
		

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "cx") == 0) 
			{	
				m_cx = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "cy") == 0) 
			{	
				m_cy = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "rx") == 0) 
			{	
				m_rx = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "ry") == 0) 
			{	
				m_ry = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "r") == 0) //occurs in the circle
			{	
				m_ry = m_rx = prs.to_length(value); //throws an exception for percents
			}
			else
				path::on_attribute(prs,name,value);
		}//on_attribute
		
		//We are able to create the path for this line, when the end of the 
		//XML source element is reached:
		virtual void commit(parser &prs) 
		{
			m_path = prs.create_ellipse(m_cx, m_cy, m_rx, m_ry);
			path::commit(prs);
		}

	protected:
		//////////////////////////////////////////////////////////////////////
		// Instance additional fields
		//////////////////////////////////////////////////////////////////////
		double m_cx, m_cy;	//center of the ellipse
		double m_rx, m_ry;	//elipse's radii
}; //line

}; //namespace svg
#endif //SVG_ELLIPSE_INCLUDED

