//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_LINEAR_TEXT_POSITION_INCLUDED
#include "svg_text_position.h"
#define SVG_LINEAR_TEXT_POSITION_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// linear_text_position: it is a helper object, used to draw text strings,
// which are not placed on a path (see svg::text element)	
//////////////////////////////////////////////////////////////////////
class linear_text_position:public text_position  
{
public:
	//constructor: creates a text position in an uninitioalized state
	linear_text_position():text_position()
	{
		m_x = m_y = 0.0;
	}

	//Sets the initial current text position
	virtual void set(double x, double y, string &text, unsigned pos, 
					unsigned count, double dx, double dy, double rotation)
	{
		text_position::set(x,y,text,pos,count,dx,dy,rotation);
		m_valid = (m_glyph != NULL); //text position is valid, when it was set at least once,
									 //and has a valid referece to the font.
		if(is_valid())
		{
			rectangle len = offset();
			m_x = x - (m_anchor * len.x2);
			m_y = y - (m_anchor * len.y2); 
		}//end if
	}
	
	//Current text position on the drawing:
	virtual double x() {return m_x;}; 
	virtual double y() {return m_y;}; 

	//actual glyph transformation
	virtual transform transformation()
	{
		transform t = internal_transform();
		t *= m_local;
		t.translate(m_x, m_y);
		return t;
	};

	//advances text position to the next point 
	//(and to next character within text chunk)
	virtual bool advance()
	{
		if (m_pos < m_count)
		{
			m_x += advance_x();
			m_y += advance_y();
			
			m_pos++;
			if (m_glyph) m_glyph->select(code(m_pos));
			//Inkscape does not take the kerning into 
			//count during advanced the character:
			if (!m_auto_kerning) //Suprising, but for the auto-kerninig
			{					 //it is not applied to the distance between
								 //letters!
				m_x += kerning_x()*m_scale;
				m_y += kerning_y()*m_scale;
			}
		}
		return (m_pos < m_count);
	}; 

protected:
		double		m_x, m_y;	//actual position on drawing
}; //linear_text_position

}; //namespace svg
#endif //SVG_LINEAR_POSITION_INCLUDED

