//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_PATH_INCLUDED
#include "svg_shape.h"
#define SVG_PATH_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// path: the <path> element of the SVG graph.
//		 It is also a base class for the specialized objects (rect, arc, ...)
//////////////////////////////////////////////////////////////////////
class path : public shape 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		path(element *parent, unsigned level): shape(parent, level) 
		{m_path = invalid_handle;}

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "d") == 0) 
			{	
				m_path = prs.parse_d(value);
			}
			else 
				{//forward this to the base class
					shape::on_attribute(prs,name,value);
				}
		}
		
		virtual const char * tag() {return "path";};

		//Default implementation of the rendering: linear text...
		virtual handle render(request &rreq)
		{
			if (!display()) return skip(rreq);

			req_geometry &req = (req_geometry &) rreq;

			//build the actual style, using the request data
			styleset	style	= req.style;
			style.update(req,m_attribute_first, m_attribute_last);

			//some final preparation: 
			//bounding box is passed to eventual painters, 
			shape::prepare_render(req, style);
			
			//render the path !
			req.perform(m_path,m_trans,style);
			
			//(inside req.render procedure final transformation is calculated: 
			//			final_transform = m_trans * req.trans
			//which ensures proper propagation of the global transform 
			//- see the forward_render() function in svg_group.h)
			return skip(req);
		}

		//Three methods required by the text on the path or the markers:

		//1. Returns a vertex source:
		vsource &vertices(req_geometry &req) {return req.paths();}

		//2. Returns the path id within this vertex source
		handle path_id() {return m_path;}

		//3. Returns the path transformation (only the individual one!)
		transform transformation() {return m_trans;}



	protected:
		handle m_path; //id of the path in the global path storage

}; //path

}; //namespace svg
#endif //SVG_PATH_INCLUDED

