//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_SVG_INCLUDED
#include "svg_group.h"
#include "svg_req_render.h"
#define SVG_SVG_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// svg: the first element (the first child of the root) 
//		of the SVG graph
//////////////////////////////////////////////////////////////////////
class svg : public group 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		svg(element *parent, unsigned level): group(parent, level) 
		{ m_x =m_y =m_width =m_height = 0; }

		virtual const char * tag() {return "svg";};

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "x") == 0) 
			{	
				m_x = prs.to_length(value);
			} else
			if (strcmp(name, "y") == 0) 
			{	
				m_y = prs.to_length(value);
			} else
			if (strcmp(name, "width") == 0) 
			{	
				m_width = prs.to_length(value);
			} else
			if (strcmp(name, "height") == 0) 
			{	
				m_height = prs.to_length(value);
			}else 
				element::on_attribute(prs,name,value);
		}

		//////////////////////////////////////////////////////////////////////
		// Other methods and fields
		//////////////////////////////////////////////////////////////////////
		//The x coordinate of the svg element 
		double x() {return m_x;}

		//The y coordinate of the svg element
		double y() {return m_y;}

		//The width of the svg element
		double width() {return m_width;}

		//The height of the svg element
		double height() {return m_height;}

		//request handler:
		virtual handle on_request(request &req) 
		{
			switch(req.type())
			{
			case request::measure:	
				return measure((req_measure &)req);

			default: 
				return group::on_request(req);
			}//switch
		}

	protected:
		//////////////////////////////////////////////////////////////////////
		// handlers of the requests - non-virtual, because virtual is 
		// the on_request() function, itself.
		//////////////////////////////////////////////////////////////////////
		handle measure(req_measure &req)
		{
			if (!display()) return skip(req);

			if (m_width > 0 && m_height > 0 && m_x == 0 && m_y == 0)
			{
				req.width = m_width;
				req.height = m_height;
			}

			return group::forward(req);
		}

	protected:
		double m_x, m_y, m_width, m_height;

}; //element

}; //namespace svg
#endif //SVG_SVG_INCLUDED

