//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef TEX_OVERRIDE_INCLUDED
#include "svg_shape.h"
#include "svg_opacity.h"
#define TEX_OVERRIDE_INCLUDED

namespace tex
{
//////////////////////////////////////////////////////////////////////
// tex::override: element NOT from the SVG spec!
// You can ignore it. It is used in an xml files that defines details
// of Blender svg textures. Used to override the style settings of these files,
// to turn on/off visibility of one of the layers in the svg file, for example.
// Used in an xml texture definition file.
//	
//////////////////////////////////////////////////////////////////////
class override : public svg::element 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		override(svg::element *parent, unsigned level): element(parent, level),
			m_href(svg::invalid_handle), 
			m_propagate(false) 
		{}

		virtual const char * tag() {return "tex:override";};

		virtual void on_attribute(svg::parser &prs, const char * name, const char *value)
		{
			svg::handle ref = svg::invalid_handle; //helper variable

			if (strcmp(name, "for") == 0) 
			{
				m_href  = prs.get_id(value); //id of the path
			}
			else
			if (strcmp(name, "propagate") == 0) 
			{
				m_propagate = (strcmp(value, "all") == 0 || strcmp(value, "1") == 0); //
			}
			else
			if (strcmp(name, "filter") == 0) 
			{	
				ref = prs.get_id(value); //id of the filter element
				if (svg::is_valid(ref)) prs.store(*this, svg::attribute(svg::attribute::filter_ref, ref));
			}
			else
			if (strcmp(name, "mask") == 0) 
			{	
				ref = prs.get_id(value); //id of the mask element
				if (svg::is_valid(ref)) prs.store(*this, svg::attribute(svg::attribute::mask_ref, ref));
			}
			else
			if (strcmp(name, "clip-path") == 0) 
			{	
				ref = prs.get_id(value); //id of the clip-path element
				if (svg::is_valid(ref)) prs.store(*this, svg::attribute(svg::attribute::clip_path_ref, ref));
			}
			else
			if (strcmp(name, "opacity") == 0) 
			{	//Opacity value is stored (as a filter) in an unnamed element
				ref = svg::opacity::create(prs,this,level()+1,prs.to_double(value));
				if (svg::is_valid(ref)) prs.store(*this, svg::attribute(svg::attribute::opacity_ref, ref));
			}
			else
			if (strcmp(name, "display") == 0) 
			{
				prs.store(*this, svg::attribute(svg::attribute::display, !(strcmp(value, "none") == 0)));
			}
			else
			{
				if(! prs.store_as_style(*this, name, value)) 
				{//if the style is not recognized:
					element::on_attribute(prs,name,value);
				}			
			}
		}//on_attribute

		//request handler:
		virtual svg::handle on_request(svg::request &req) 
		{
			switch(req.type())
			{
			case svg::request::initialize:	
				return initialize(req);
			default: 
				return element::on_request(req);
			}//switch
		}//on_request
	
		//////////////////////////////////////////////////////////////////////
		// Other methods and fields
		//////////////////////////////////////////////////////////////////////
	protected:
		svg::handle initialize(svg::request &req)
		{
			if (!svg::is_valid(m_attribute_last)) return skip(req); //no attributes,
																	  //nothing to change

			svg::shape *elm = (svg::shape *) req.get_element(m_href,"shape");
			
			if (elm) //found?
			{
				svg::req_override rovr(m_propagate);
				svg::req_prepare prepare(svg::request::prepare, rovr);
				elm->to_root(prepare);
				if(rovr.can_query())
				{
					svg::handle aid;
					//send to the target each defined attribute
					for(aid = m_attribute_first; aid <= m_attribute_last; aid++)
					{
						svg::attribute *attr = req.get_attribute(aid);
						if(attr)
						{
							rovr.value = *attr;
							elm->on_request(rovr);
						}//end if(attr)
					}//end for
				}//end if
			}//end if

			return skip(req);
		}//end initialize

	protected:
		svg::handle m_href;		//handle of the target shape
		bool m_propagate;		//propagate this setting down to the children?
}; //override

}; //namespace tex
#endif //TEX_OVERRIDE_INCLUDED

